/******************************************************************************
*                                                                             *
*   t i c o n s t                                                             *
*                                                                             *
*   by Stefan Heule                                                           *
*   http://stefanheule.com                                                    *
*                                                                             *
*   Source Code                                                               *
*                                                                             *
*******************************************************************************

ticonst - replacement for the standard units menu
Copyright (C) 2009-2010 Stefan Heule

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/



// ----------------------------------------------------------------- //
// --- header files
// ----------------------------------------------------------------- //
#include "header.h"





// ----------------------------------------------------------------- //
// loads the configuration from a file
// ----------------------------------------------------------------- //
// local:
//     backup = config structure
//     file   = file structure
//     chcksm = file checksum
//     backup_file_magic = file magic
// return:
//     TRUE, or, in case of an error, FALSE
// note:
//     If there's no config file, a new one with default values will
//     be created
// ----------------------------------------------------------------- //
short loadBackup(void)
{
	OPTIONS backup;
	FILES file;
	unsigned char chcksm;
	unsigned char backup_file_magic[strlen(OPTIONS_FILE_MAGIC)+1];
	
	// unarchive the file
	EM_moveSymFromExtMem(SYMSTR(OPTIONS_FILE_NAME), HS_NULL);
	
	// open the backup file
	if (FOpen(OPTIONS_FILE_NAME, &file, FM_READ, OPTIONS_FILE_EXT) != FS_OK)
	{
		return TRUE;
	}
	
	// read the file magic
	backup_file_magic[strlen(OPTIONS_FILE_MAGIC)] = 0;
	if (FRead((void*)backup_file_magic, strlen(OPTIONS_FILE_MAGIC), &file) != FS_OK)
	{
		FClose(&file);
		ST_helpMsg(MSG_ERROR_READ_FROM_FILE);
		return FALSE;
	}
	
	// check file version
	if (strcmp(OPTIONS_FILE_MAGIC, backup_file_magic) != 0)
	{
		FClose(&file);
		ST_helpMsg(MSG_ERROR_OUTDATED_FILE);
		return FALSE;
	}
	
	// read the file content
	if (FRead((void*)&backup, sizeof(OPTIONS), &file) != FS_OK)
	{
		FClose(&file);
		ST_helpMsg(MSG_ERROR_READ_FROM_FILE);
		return FALSE;
	}
	
	// read the checksum
	if (FRead((void*)&chcksm, 1, &file) != FS_OK)
	{
		FClose(&file);
		ST_helpMsg(MSG_ERROR_READ_FROM_FILE);
		return FALSE;
	}
	
	// check checksum
	if (checksum((void*)&backup, sizeof(OPTIONS)) != chcksm)
	{
		FClose(&file);
		ST_helpMsg(MSG_ERROR_CORRUPTED_FILE);
		return FALSE;
	}
	
	// close the file
	FClose(&file);
	
	pref = backup;
	
	// archive the file
	EM_moveSymToExtMem(SYMSTR(OPTIONS_FILE_NAME), HS_NULL);
	
	return TRUE;
}


// ----------------------------------------------------------------- //
// calculates a simple 1-byte-checksum of the provided data
// ----------------------------------------------------------------- //
// local:
//     md5 = MD5 structure
//     res = 16 byte MD5 checksum
// return:
//     first byte of MD5 checksum
// note:
//     only one byte of the 16-byte MD5 checksum will be used
// ----------------------------------------------------------------- //
unsigned char checksum(void* data, unsigned short bytes)
{
	unsigned char res[16];
	MD5_CTX md5;
	
	// get MD5 hash of the data
	MD5Init(&md5);
	MD5Update(&md5, data, bytes);
	MD5Final(res, &md5);
	
	// we just use the first byte of the 16 byte long MD5 hash
	return res[0];
}


// ----------------------------------------------------------------- //
// saves the configuration ot a file
// ----------------------------------------------------------------- //
// local:
//     file = file structure
// return:
//     TRUE, or, in case of an error, FALSE
// note:
//     Any old file will be overwritten
// ----------------------------------------------------------------- //
short saveBackup(void)
{
	FILES file;
	
	// unarchive the file
	EM_moveSymFromExtMem(SYMSTR(OPTIONS_FILE_NAME), HS_NULL);
	
	// open the backup file
	if (FOpen(OPTIONS_FILE_NAME, &file, FM_WRITE, OPTIONS_FILE_EXT) != FS_OK)
	{
		ST_helpMsg(MSG_ERROR_CREATE_FILE);
		return FALSE;
	}
	
	// write the file magic
	if (FWrite((void*)OPTIONS_FILE_MAGIC, strlen(OPTIONS_FILE_MAGIC), &file) != FS_OK)
	{
		FClose(&file);
		ST_helpMsg(MSG_ERROR_WRITE_TO_FILE);
		return FALSE;
	}
	
	// write backup
	if (FWrite((void*)&pref, sizeof(OPTIONS), &file) != FS_OK)
	{
		FClose(&file);
		ST_helpMsg(MSG_ERROR_WRITE_TO_FILE);
		return FALSE;
	}
	
	// calculate checksum
	unsigned char chcksm = checksum((void*)&pref, sizeof(OPTIONS));
	
	// write the checksum
	if (FWrite((void*)&chcksm, 1, &file) != FS_OK)
	{
		FClose(&file);
		ST_helpMsg(MSG_ERROR_WRITE_TO_FILE);
		return FALSE;
	}
	
	FClose(&file);
	
	// archive the file
	EM_moveSymToExtMem(SYMSTR(OPTIONS_FILE_NAME), HS_NULL);
	
	return TRUE;
}


// ----------------------------------------------------------------- //
// delete preference file
// ----------------------------------------------------------------- //
// ----------------------------------------------------------------- //
void deleteBackup(void)
{
	// unarchive the file
	EM_moveSymFromExtMem(SYMSTR(OPTIONS_FILE_NAME), HS_NULL);
	FDelete(OPTIONS_FILE_NAME);
}























